/* Txt282p by Kouri (Kouri@ucdavis.edu)
 * http://kouri.cjb.net
 * 1999.12.25

Program Information:
Converts an ASCII text file into a TI-82 program file. Keeping notes on
your calculator is now easier than ever. For example, let's say you have a
ton of vocabulary words you have to memorize for a certain class. Type them
up on your computer, run the file through Txt282p, and send it to your
calc. Use TxtView to view the file on your calc. See TxtView.asm for more
information.

Usage:
Type "txt282p <infile> [<outfile> ["<description>"]]" at the DOS prompt.
<infile> is the name of the file that you wish to convert.
<outfile> is the name of the output file.
If not outfile is provided, <infile.82p> will be used instead.
<description> can be up to 24 characters and should be enclosed with
quotes. It will be shown in TxtView's Select File menu. If no description
is given, <outfile> will be used instead.
Example: txt282p vocab.txt "Chapter 16 Physics Vocab"
*Note* Put Txt282p in a directory in your path statement for easiest usage.

Credits: Thomas Hruska, for putting up with all my e-mail asking him for
help and for actually taking the time to help me.
Dan Eble, for releasing the source to string85.exe.
Dines Justesen, for also helping me out.
Doug Torrence, for his Hacking the TI-82 columns. */

#include <stdio.h>
#include <string.h>

#define DESCRIPTION_LENGTH 24

/* Function declarations */
void fputc_and_sum(int);
char *strccpy(const char *, const char *, int);

/* Global variable declarations */
FILE *infile, *outfile;
unsigned long checksum = 0;

int main(int argc, char *argv[])
{
 int c;
 unsigned long i;
 char filename[80], description[DESCRIPTION_LENGTH+1];

 printf("\nText to 82p file converter - http://kouri.cjb.net\n");

 switch (argc)
 {
  case 2:
   strccpy(filename, argv[1], '.');		/* Copy up to '.' */
   strncpy(description, argv[1], DESCRIPTION_LENGTH);
   break;

  case 3:
   strccpy(filename, argv[2], '.');
   strncpy(description, argv[2], DESCRIPTION_LENGTH);
   break;

  case 4:
   strccpy(filename, argv[2], '.');
   strncpy(description, argv[3], DESCRIPTION_LENGTH);
   break;

  default:
   fprintf(stderr, "Usage: txt282p <infile> [<outfile> [\"<description>\"]]\n");
   fprintf(stderr, "<infile> is the text file you wish to convert.\n");
   fprintf(stderr, "<outfile> is the name of the output file.\n");
   fprintf(stderr, "If no outfile is provided, <infile.82p> will be used instead.\n");
   fprintf(stderr, "<description> can be up to 24 characters and should be enclosed with quotes.\n");
   fprintf(stderr, "If no description is provided, <outfile> will be used instead.\n");
   return 1;
   break;
 }

 infile = fopen(argv[1], "r");
 if (infile == NULL)
 {
  fprintf(stderr, "Error: Can't read file: %s\n", argv[1]);
  return 2;
 }

 strcat(filename, ".82p");
 outfile = fopen(filename, "wb");
 if (outfile == NULL)
 {
  fprintf(stderr, "Error: Can't write file: %s\n", filename);
  return 3;
 }

/* Just making sure
 printf("filename: %s (length %d)\n", filename, strlen(filename));
 printf("description: %s (length %d)\n", description, strlen(description));
 return 0;
*/

/* Calculate length of file */
 for (i = 5 + strlen(description); fgetc(infile) != EOF; i++);

/* Write header */
 fputs("**TI82**", outfile);
 fputc(0x1A, outfile);
 fputc(0x0A, outfile);
 fputc(0x00, outfile);
 fputs("TxtView file generated by Txt282p by Kouri", outfile);
 fputc((i+17) & 0xFF, outfile);
 fputc((i+17) >> 8, outfile);
 fputc_and_sum(0x0B);
 fputc(0x00, outfile);
 fputc_and_sum((i+2) & 0xFF);
 fputc_and_sum((i+2) >> 8);
 fputc_and_sum(0x06);

/* Write filename */
 for (c = 0; (c < 8) && (filename[c] != '.'); c++)
  fputc_and_sum(filename[c]);
 for (; c < 8; c++)
  fputc(0x00, outfile);

/* Write more data */
 fputc_and_sum((i+2) & 0xFF);
 fputc_and_sum((i+2) >> 8);
 fputc_and_sum(i & 0xFF);
 fputc_and_sum(i >> 8);

/* Write file ID used by TxtView */
 fputc_and_sum(0xFF);
 fputc(0x00, outfile);

/* Write file description */
 for (c = 0; (c < DESCRIPTION_LENGTH) && (description[c] != NULL); c++)
  fputc_and_sum(description[c]);
 fputc(0x00, outfile);

/* Write actual text */
 rewind(infile);
 while ((c = fgetc(infile)) != EOF)
 {
  if (c == '\n') fputc(0x00, outfile);
  else fputc_and_sum(c);
 }

/* Write EOF used by TxtView */
 fputc(0x00, outfile);
 fputc_and_sum(0xFF);

/* Write checksum */
 fputc(checksum & 0xFF, outfile);
 fputc(checksum >> 8, outfile);

/* And we're done! */
 printf("File written: %s\n", filename);
 return 0;
}

/* Function taken from string85 by Dan Eble - THANKS! */
void fputc_and_sum(c)
{
 fputc(c, outfile);
 checksum += c;
}

/* Function to copy string2 into string1, until character c is reached */
char *strccpy(const char *string1, const char *string2, int c)
{
 int i;
 for(i = 0; (string2[i] != c) && (string2[i] != NULL); i++)
  string1[i] = string2[i];
 return string1;
}